
#ifndef INTERFACE_H_
#define INTERFACE_H_

#include<string>
#include<map>
#include<list>
#include<vector>

using namespace std;

enum ArgArg{
  arg_no_arg = 1,
  arg_opt_arg = 2,
  arg_req_arg = 4
};

struct ArgsDef {
  char shortSwitch;
  string longSwitch;
  ArgArg arg;  
  string help;
  string arg_regex;
};

const ArgsDef args_allinone[] = {
  {'v', "verbose", arg_opt_arg, "select verbosity of the program; values:"
                                " [error, warning, notice, info, debug]", 
                                "error|warning|notice|info|debug"},
  {'O', "outdir", arg_req_arg, "select output dir", "error|warning|notice|info"},
  {'h', "help", arg_no_arg, "print this help message", ""},
  {'U', "upper", arg_no_arg, "Change names to uppercase", ""},
  {'u', "lower", arg_no_arg, "Change names to lowercase", ""},
  {0, "", arg_no_arg, "", ""}
};

const ArgsDef args_common[] = {
  {'p', "package", arg_req_arg, "select the package"},
  {'i', "input", arg_req_arg, "select select input [file|{package[:packge..]}|\\<package number>]",""},
  {'o', "output", arg_req_arg, "select output file for the results",""},
  {'n', "number-of", arg_req_arg, "number of created ouptuts",""},
  {0, "", arg_no_arg, "", ""}
};

struct InterfaceFlow{
  enum InterfaceArg{
    None = 0x1,
    Microarray = 0x2,
    Results = 0x4,
    Multiple = 0x8,
    Graph = 0x10,
    All = 0x1 | 0x2 | 0x4 | 0x8 | 0x10 
  };
  InterfaceArg input, output;
  InterfaceFlow() : input(None), output(None) {}; 
  InterfaceFlow(InterfaceArg input, InterfaceArg output) : input(input), output(output) {}; 
};

class Interface{
  public:
  InterfaceFlow::InterfaceArg arg;
  bool readOnly;
  bool freeAble;
  Interface(InterfaceFlow::InterfaceArg arg) : arg(arg), 
    readOnly(false), freeAble(true) {}; 
  Interface() : arg(InterfaceFlow::None), 
    readOnly(false), freeAble(true) {}; 
  Interface(const Interface * i) : 
    arg(i->arg), readOnly(i->readOnly), freeAble(i->freeAble) {}; 
//  virtual void dummy() = 0;
};



enum Verbosity{
  NONE = 0,
  ERROR = 1,
  WARNING = 2,
  NOTICE = 3,
  INFO = 4,
  DEBUG = 5
};

namespace allinone{
  const pair<const int, const char * const> tmpArgAtomType2Str[] = { 
     pair<const int, const char * const>(0x1,"ShortSwitch"),
     pair<const int, const char * const>(0x2,"LongSwitch"),
     pair<const int, const char * const>(0x4,"Arg"),
     pair<const int, const char * const>(0x8,"Operator"),
     pair<const int, const char * const>(0x10,"PackageNumber"),
     pair<const int, const char * const>(0x20,"ToProcess"),
     pair<const int, const char * const>(0x40,"None")
   };
  const map<const int, const char * const> ArgAtomType2Str
  (tmpArgAtomType2Str, 
    &tmpArgAtomType2Str[sizeof(tmpArgAtomType2Str)/sizeof(*tmpArgAtomType2Str)]);

  struct ArgAtom;
  typedef list<ArgAtom *>  ArgAtoms;
  struct Package;   
  typedef list<Package *>  Packages;
  typedef vector<Package *> PackagesByNumbers;

  #define InterfaceFunction int process(allinone::Package * package)

  #define InterfaceFunctionBegin                                              \
    InterfaceFunction{                                                        \
      if(package->interface && package->interface->arg != InterfaceFlow::None)\
        return 0;                                                             \
      int n = 1;                                                              \
      char str[100];                                                          \
      sprintf(str, "%s:p[%d]", package->name.c_str(), package->packageNumber);\
      timer::Timer t(str);                                                    \
      t.start();
  
  #define InterfaceFunctionEnd                                                \
      t.stop();                                                               \
      return 0;                                                               \
    }

  typedef int (*InterfaceFunctionPtr)(allinone::Package *);

  typedef pair<const string, const ArgsDef *> TmpPackageArgs;
  typedef pair<const string, InterfaceFunctionPtr> TmpPackageInterface;
  typedef pair<const string, const InterfaceFlow *> TmpInterfaceFlow;

  typedef const map<const string, const ArgsDef *> PackageArgs;
  typedef const map<const string, const InterfaceFunctionPtr> PackageInterface;
  typedef const map<const string, const InterfaceFlow *> InterfacesFlow;


  struct ArgAtom {
    enum ArgAtomType { ShortSwitch = 0x1, LongSwitch = 0x2, Arg = 0x4, Operator = 0x8, PackageNumber = 0x10, ToProcess = 0x20, None = 0x40 }; 
    ArgAtomType argAtomType;
    string value;
    Packages packages;
    ArgAtom() : argAtomType(None) { }
    ArgAtom(const ArgAtom & atom) : argAtomType(atom.argAtomType), value(atom.value),
                                    packages(atom.packages) { }
    ArgAtom(const string & name) : argAtomType(None), value(name) { }

    ArgAtom(ArgAtomType argAtomType, const string & name = "") : 
                          argAtomType(argAtomType), value(name){}

    ArgAtom(const Packages & p) : argAtomType(ToProcess), packages(p){}

    bool isPackageSpecifier(){
      return (argAtomType == ShortSwitch && value.substr(1) == "p" ||
              argAtomType == LongSwitch && value.substr(2) == "package");
    }
  };
  

  struct Package {
    ArgAtoms argNode;
    string name;
    int packageNumber;
    Interface * interface;
    Package(ArgAtoms::iterator beg, ArgAtoms::iterator end, int & PackageNumber, const string & str = "");
    void checkConsistency(int level);
  };

  
  extern ArgAtoms argsAtomised;
  extern Packages packages;
  extern PackagesByNumbers packagesByNumbers;
  extern string outDir;
  extern enum ChangeCase { no, upper, lower } changeCase;

  void atomizeArguments(const int argc, const char ** argv);
}


#define _STRINGIFY(X) #X
#define STRINGIFY(X) _STRINGIFY(X)

#include "config.h" 
#include <boost/preprocessor/seq.hpp>   

#define CREATE_ARG_SYNTAX(r, f, t) TmpPackageArgs(string(STRINGIFY(t)), BOOST_PP_SEQ_CAT((args_)(t))),
#define CREATE_INTERFACE_SYNTAX(r, f, t) TmpPackageInterface(string(STRINGIFY(t)), t::process),
#define CREATE_INTERFACEARG_SYNTAX(r, f, t) TmpInterfaceFlow(string(STRINGIFY(t)), &t::interfaceFlow),

namespace allinone { 

  InterfaceFunction;

  const TmpPackageArgs tmpPackageArgs[] = {
    BOOST_PP_SEQ_FOR_EACH(CREATE_ARG_SYNTAX, ~, PACKAGES(allinone)(common))
    TmpPackageArgs("", 0)
  };

  const TmpPackageInterface tmpPackageInterface[] = {
    BOOST_PP_SEQ_FOR_EACH(CREATE_INTERFACE_SYNTAX, ~, PACKAGES(allinone))
    TmpPackageInterface("", 0)
  };

  const TmpInterfaceFlow tmpInterfacesFlow[] = {
    BOOST_PP_SEQ_FOR_EACH(CREATE_INTERFACEARG_SYNTAX, ~, PACKAGES)
    TmpInterfaceFlow("", 0)
  };
  
  
  PackageArgs packageArgs(tmpPackageArgs, 
        &tmpPackageArgs[sizeof(tmpPackageArgs) / sizeof(TmpPackageArgs) - 1]);

  PackageInterface packageInterface(
        tmpPackageInterface, &tmpPackageInterface[sizeof(tmpPackageInterface) 
                                        / sizeof(TmpPackageInterface) - 1]);
  
  InterfacesFlow interfacesFlow(
    tmpInterfacesFlow, &tmpInterfacesFlow[sizeof(tmpInterfacesFlow) 
                                  / sizeof(TmpInterfaceFlow) - 1]);

}
extern Verbosity verbosity;

#ifndef HAVE_LOGGER_INTERFACE_H
#define LOG(err, str, ...)                                                    \
        do{                                                                   \
         if(err == ERROR){                                                    \
           fprintf(stderr, str, __VA_ARGS__) ;                                \
           exit (-1);                                                         \
         }                                                                    \
         while(0)
#endif

#endif /*INTERFACE_H_*/
